package com.sillycycle.bagleyd.life3d;

/*
 * @(#)Life3DSoup.java
 *
 * Copyright 1993 - 2017  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.util.Random;

/**
 * The <code>Life3DSoup</code> class does all the random cells for
 * <code>Life3DCanvas</code> class.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/life.html
 */

/* This may help getting all possible symmetries: */
/* http://en.wikipedia.org/wiki/Point_groups_in_three_dimensions */

public class Life3DSoup {
	static Life3DCanvas canvas = null;
	Random generator;

	static final String[] symmetryStrings = {
		"Full Random",
		"Sym X Random",
		"Sym Y Random",
		"Sym XY Random",
		"Antisym X Random",
		"Antisym Y Random",
		"Half Antisym Random", // Diagonal
		"Half Sym Random", // Mirror Diagonal
		"Full Antisym Random", // Square
		"Full Sym Random", // Mirror Square
		"Half Antisym Skew Random",
		"Alt Skew Antisym Random",
		"Antisym Z Random",
		/* Non Glider Patterns */
		"Sym XYZ Random",
		"Half Sym Skew Random",
		"Full Antisym Cube Random",
		"Full Sym Cube Random",
		"Full Tetra Random",
		"Alt Full Antisym Cube Random",
		"Propeller Random",
		"New One Random",
	};

	public Life3DSoup(Life3DCanvas canvas) {
		Life3DSoup.canvas = canvas;
		generator = new Random(System.nanoTime());

	}

	static Integer stringToSymmetryIndex(String string) {
		for (int i = 0; i < symmetryStrings.length; i++) {
			if (symmetryStrings[i].equals(string)) {
				return i;
			}
		}
		return null;
	}

	static int soupSize(int s) {
		// 14 needs this...
		int val = s / 12;
		int bound = 6;
		int polyhedron = canvas.getPolyhedron();
		if (polyhedron == 4 || polyhedron == 8 || polyhedron == 16) {
			val = s / 6;
			bound = 8;
		}
		return ((val > bound) ? val : bound);
	}

	public int randomSymmetry(boolean gliderSearch) {
		// For glider searches, too much symmetry can cause the
		// gliders to be short lived since wrap is used to find them.
		final int vector = 13;

		if (gliderSearch) {
			return generator.nextInt(vector); /* all 2D ones + Skew Diagonal */
		}
		//return generator.nextInt(symmetryStrings.length - 1) + 1;
		return generator.nextInt(symmetryStrings.length);
		//return generator.nextInt(symmetryStrings.length - vector) + vector - 1;
		//return generator.nextInt(symmetryStrings.length);
		//return 0;
		//return 18;
		//return 19; // Experimental
		//return generator.nextInt(symmetryStrings.length - vector) + vector;
	}

	public void displayRandomSoup(int symmetryIndex, int percentChance, int stirs,
			int fieldX, int fieldY, int fieldZ) {
		int polyhedron = canvas.getPolyhedron();
		int ncols = canvas.getColumns();
		int nrows = canvas.getRows();
		int nstacks = canvas.getStacks();
		int col, row, stack;
		int halfcols = ncols / 2;
		int halfrows = nrows / 2;
		int halfstacks = nstacks / 2;
		int vx = fieldX / 2 + 1;
		int vy = fieldY / 2 + 1;
		int vz = fieldZ / 2 + 1;
		int even = 0;
		int yEven = 0;
		int zEven = 0;
		int zDEven = 0;
		int factor = 1;
		int state = 1;
		int percent = percentChance;

		if (vx < 1)
			vx = 1;
		if (vy < 1)
			vy = 1;
		if (vz < 1)
			vz = 1;
		//name = symmetryStrings[symmetryIndex];
		if (stirs > 0) {
			/* make smaller as it should expand */
			vx = (fieldX / 3) | 1;
			vy = (fieldY / 3) | 1;
			if (polyhedron == 14) {
				vz = ((fieldZ / 3) * 2) | 1;
			} else {
				vz = (fieldZ / 3) | 1;
			}
		} else if (polyhedron == 14) {
			vz = (fieldZ / 2) * 2 + 1;
		}
		if (polyhedron != 12 && polyhedron != 14 &&
				polyhedron != 4 && polyhedron != 8 && polyhedron != 16) {
			even = generator.nextInt(2);
			yEven = generator.nextInt(2);
			zEven = generator.nextInt(2);
		}
		if (polyhedron == 4 || polyhedron == 8 || polyhedron == 16) {
			zDEven = 2 * generator.nextInt(2);
		}

		//System.out.println("symmetryIndex = " + symmetryIndex);
		//System.out.println(halfcols + " " + halfrows  + " " + halfstacks + " " +  even);
		switch (symmetryIndex) {
		case 0: /* Full Random */
			for (stack = halfstacks - vz; stack < halfstacks + vz; ++stack)
			for (row = halfrows - vy; row < halfrows + vy; ++row)
			for (col = halfcols - vx; col < halfcols + vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack) &&
					(generator.nextInt(100) < percent)) {
				canvas.setCell(col, row, stack, state);
			}
			break;
		case 1: /* Sym X Random */
			for (stack = halfstacks - vz; stack < halfstacks + vz; ++stack)
			for (row = halfrows - vy; row < halfrows + vy; ++row)
			for (col = halfcols; col < halfcols + vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)) {
					if (polyhedron == 14) {
						even = (stack & 1);
					}
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col, row, stack, state);
						canvas.setCell(2 * halfcols - col - even, row, stack, state);
					}
			}
			break;
		case 2: /* Sym Y Random */
			for (stack = halfstacks - vz; stack < halfstacks + vz; ++stack)
			for (row = halfrows; row < halfrows + vy; ++row)
			for (col = halfcols - vx; col < halfcols + vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)) {
					if (polyhedron == 14) {
						even = (stack & 1);
					}
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col, row, stack, state);
						canvas.setCell(col, 2 * halfrows - row - even, stack, state);
					}
			}
			break;
		case 3: /* Sym XY Random */
			for (stack = halfstacks - vz; stack < halfstacks + vz; ++stack)
			for (row = halfrows; row < halfrows + vy; ++row)
			for (col = halfcols; col < halfcols + vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)) {
					if (polyhedron == 14) {
						yEven = even = (stack & 1);
					}
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col, row, stack, state);
						canvas.setCell(col, 2 * halfrows - row - yEven, stack, state);
						canvas.setCell(2 * halfcols - col - even, row, stack, state);
						canvas.setCell(2 * halfcols - col - even, 2 * halfrows - row - yEven, stack, state);
					}
			}
			break;
		case 4: /* Antisym X Random */
			for (stack = halfstacks - vz; stack < halfstacks + vz; ++stack)
			for (row = halfrows - vy; row < halfrows + vy; ++row)
			for (col = halfcols; col < halfcols + vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)) {
					if (polyhedron == 14) {
						yEven = even = (stack & 1);
					}
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col, row, stack, state);
						canvas.setCell(2 * halfcols - col - even, 2 * halfrows - row - yEven, stack, state);
					}
			}
			break;
		case 5: /* Antisym Y Random */
			for (stack = halfstacks - vz; stack < halfstacks + vz; ++stack)
			for (row = halfrows; row < halfrows + vy; ++row)
			for (col = halfcols - vx; col < halfcols + vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)) {
					if (polyhedron == 14) {
						yEven = even = (stack & 1);
					}
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col, row, stack, state);
						canvas.setCell(2 * halfcols - col - even, 2 * halfrows - row - yEven, stack, state);
					}
			}
			break;
		case 6: /* Half Antisym Random */
			// Diagonal
			vx = vy = (vx + vy) / 2;
			for (stack = halfstacks - vz; stack < halfstacks + vz; ++stack)
			for (row = -vy; row < vy; ++row)
			for (col = row; col < vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)) {
					int extra = 0;

					if (polyhedron == 14) {
						extra = (stack & 1);
					}
					if (generator.nextInt(100) < percent) {
						if (even == 1) {
							canvas.setCell(col + halfcols, row + halfrows, stack, state);
							canvas.setCell(row + halfcols, col + halfrows, stack, state);
						} else {
							canvas.setCell(-col + halfcols - extra, row + halfrows, stack, state);
							canvas.setCell(-row + halfcols - extra, col + halfrows, stack, state);
						}
					}
			}
			break;
		case 7: /* Half Sym Random */
			// Diagonal (on face, on corner)
			vx = vy = (vx + vy) / 2;
			for (stack = halfstacks - vz; stack < halfstacks + vz; ++stack)
			for (row = -vy; row < 0; ++row)
			for (col = row; col < vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)) {
					int extra = 0;

					if (polyhedron == 14) {
						extra = (stack & 1);
					}
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col + halfcols, row + halfrows, stack, state);
						canvas.setCell(row + halfcols, col + halfrows, stack, state);
						canvas.setCell(-col + halfcols - extra, -row + halfrows - extra, stack, state);
						canvas.setCell(-row + halfcols - extra, -col + halfrows - extra, stack, state);
					}
			}
			break;
		case 8: /* Full Antisym Random */
			// Square
			vx = vy = (vx + vy) / 2;
			for (stack = halfstacks - vz; stack < halfstacks + vz; ++stack) {
				int extra = 0;

				if (polyhedron == 14) {
					extra = (stack & 1);
				}
				for (row = -vy; row < 0; ++row)
				for (col = row; col < vx; ++col)
				if (Life3DCanvas.cellCheck(polyhedron, col, row, stack) &&
						(generator.nextInt(100) < percent)) {
					canvas.setCell(col + halfcols + even, row + halfrows + even, stack, state);
					canvas.setCell(row + halfcols + even, -col + halfrows - extra, stack, state);
					canvas.setCell(-col + halfcols - extra, -row + halfrows - extra, stack, state);
					canvas.setCell(-row + halfcols - extra, col + halfrows + even, stack, state);
				}
				if (Life3DCanvas.cellCheck(polyhedron, halfcols, halfrows, stack) &&
						even == 0 && generator.nextInt(100) < percent) {
					canvas.setCell(halfcols, halfrows, stack, state);
				}
			}
			break;
		case 9: /* Full Sym Random */
			// Square
			vy = (vx + vy) / 2;
			for (stack = halfstacks - vz; stack < halfstacks + vz; ++stack) {
				int extra = 0;

				if (polyhedron == 14) {
					extra = (stack & 1);
				}
				for (row = -vy; row < 0; ++row)
				for (col = row; col <= 0; ++col)
				if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)
						&& generator.nextInt(100) < percent) {
					canvas.setCell(col + halfcols + even, row + halfrows + even, stack, state);
					canvas.setCell(row + halfcols + even, -col + halfrows - extra, stack, state);
					canvas.setCell(-col + halfcols - extra, -row + halfrows - extra, stack, state);
					canvas.setCell(-row + halfcols - extra, col + halfrows + even, stack, state);
					canvas.setCell(-col + halfcols - extra, row + halfrows + even, stack, state);
					canvas.setCell(-row + halfcols - extra, -col + halfrows - extra, stack, state);
					canvas.setCell(col + halfcols + even, -row + halfrows - extra, stack, state);
					canvas.setCell(row + halfcols + even, col + halfrows + even, stack, state);
				}
				if (Life3DCanvas.cellCheck(polyhedron, halfcols, halfrows, stack)
						&& even == 0 && generator.nextInt(100) < percent) {
					canvas.setCell(halfcols, halfrows, stack, state);
				}
			}
			break;
		case 10: /* Half Antisym Skew Random */
			// Skew Diagonal
			vx = vy = vz = (vx + vy + vz) / 3;
			factor = 1;
			if (polyhedron == 14) {
				factor = 2;
			for (stack = -vz; stack < vz - 1; ++stack)
			for (row = stack; row < vy; ++row)
			for (col = row; col < vx; ++col) {
					if (generator.nextInt(100) < percent) {
						if (even == 0) {
							canvas.setCell(col + halfcols, row + halfrows, factor * stack + 1 + halfstacks, state);
							canvas.setCell(row + halfcols, stack + halfrows, factor * col + 1 + halfstacks, state);
							canvas.setCell(stack + halfcols, col + halfrows, factor * row + 1 + halfstacks, state);
						} else {
							canvas.setCell(-col + halfcols - 1, row + halfrows, factor * stack + 1 + halfstacks, state);
							canvas.setCell(-row + halfcols - 1, stack + halfrows, factor * col + 1 + halfstacks, state);
							canvas.setCell(-stack + halfcols - 1, col + halfrows, factor * row + 1 + halfstacks, state);
						}
					}
			}
			}
			for (stack = -vz; stack < vz; ++stack)
			for (row = stack; row < vy; ++row)
			for (col = row; col < vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, -col + halfcols, row + halfrows, factor * stack + halfstacks - zDEven)
					&& generator.nextInt(100) < percent) {
				if (even == 0) {
					canvas.setCell(col + halfcols, row + halfrows, factor * stack + halfstacks - zDEven, state);
					canvas.setCell(row + halfcols, stack + halfrows, factor * col + halfstacks - zDEven, state);
					canvas.setCell(stack + halfcols, col + halfrows, factor * row + halfstacks - zDEven, state);
				} else {
					canvas.setCell(-col + halfcols, row + halfrows, factor * stack + halfstacks - zDEven, state);
					canvas.setCell(-row + halfcols, stack + halfrows, factor * col + halfstacks - zDEven, state);
					canvas.setCell(-stack + halfcols, col + halfrows, factor * row + halfstacks - zDEven, state);
				}
			}
			break;
		case 11: /* Alt Skew Sym Random */
			int offset = generator.nextInt(2);
			if (polyhedron == 4 || polyhedron == 8 || polyhedron == 16)
				offset = 2;
			for (stack = -vz; stack < 0; ++stack)
			for (row = -vy; row < 0; ++row)
			for (col = -vx; col < 0; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even - zDEven)
					&&generator.nextInt(100) < percent) {
				canvas.setCell(col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even - zDEven, state);
				canvas.setCell(-stack + halfcols, -row + halfrows + offset, -factor * col + halfstacks + offset - zDEven, state);
			}
			break;
		case 12: /* Antisym Z Random */
			int which = generator.nextInt(3);
			if (polyhedron == 12) {
				even = yEven = generator.nextInt(2);
			}
			for (stack = halfstacks - vz; stack < halfstacks + vz; ++stack)
			for (row = halfrows; row < halfrows + vy; ++row)
			for (col = halfcols - vx; col < halfcols + vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack - zDEven)
					&& generator.nextInt(100) < percent) {
				canvas.setCell(col, row, stack - zDEven, state);
				if (which == 0) {
					canvas.setCell(2 * halfcols - col - even, 2 * halfrows - row - yEven, 2 * halfstacks - stack - zDEven, state);
				} else if (which == 1) {
					canvas.setCell(2 * halfcols - col - even, 2 * halfstacks - row - yEven, stack - zDEven, state);
				} else {
					canvas.setCell(2 * halfcols - col - even, row + yEven, stack - zDEven, state);
				}
			}
			break;
		case 13: /* Sym XYZ Random */
			//halfstacks = 2 * (nstacks / 4);
			if (polyhedron == 14)
			for (stack = halfstacks; stack < halfstacks + vz - 2; ++stack)
			for (row = halfrows; row < halfrows + vy - 1; ++row)
			for (col = halfcols; col < halfcols + vx - 1; ++col)
			if ((stack & 1) == 1 && generator.nextInt(100) < percent) {
				canvas.setCell(col, row, stack, state);
				canvas.setCell(col, 2 * halfrows - row - yEven - 1, stack, state);
				canvas.setCell(2 * halfcols - col - even - 1, row, stack, state);
				canvas.setCell(2 * halfcols - col - even - 1, 2 * halfrows - row - yEven - 1, stack, state);
				canvas.setCell(col, row, 2 * halfstacks - stack - zEven, state);
				canvas.setCell(col, 2 * halfrows - row - yEven - 1, 2 * halfstacks - stack - zEven, state);
				canvas.setCell(2 * halfcols - col - even - 1, row, 2 * halfstacks - stack - zEven, state);
				canvas.setCell(2 * halfcols - col - even - 1, 2 * halfrows - row - yEven - 1, 2 * halfstacks - stack - zEven, state);
			}
			for (stack = halfstacks; stack < halfstacks + vz; ++stack)
			for (row = halfrows; row < halfrows + vy; ++row)
			for (col = halfcols; col < halfcols + vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack - zDEven) &&
					(polyhedron != 14 || (stack & 1) == 0) && generator.nextInt(100) < percent) {
				canvas.setCell(col, row, stack - zDEven, state);
				canvas.setCell(col, 2 * halfrows - row - yEven, stack - zDEven, state);
				canvas.setCell(2 * halfcols - col - even, row, stack - zDEven, state);
				canvas.setCell(2 * halfcols - col - even, 2 * halfrows - row - yEven, stack - zDEven, state);
				canvas.setCell(col, row, 2 * halfstacks - stack - zEven - zDEven, state);
				canvas.setCell(col, 2 * halfrows - row - yEven, 2 * halfstacks - stack - zEven - zDEven, state);
				canvas.setCell(2 * halfcols - col - even, row, 2 * halfstacks - stack - zEven - zDEven, state);
				canvas.setCell(2 * halfcols - col - even, 2 * halfrows - row - yEven, 2 * halfstacks - stack - zEven - zDEven, state);
			}
			break;
		case 14: /* Half Sym Skew Random */
			// Skew Diagonal (on face, on corner)
			vx = vy = vz = (vx + vy + vz) / 3;
			if (polyhedron == 14) {
				factor = 2;
			for (stack = -vz; stack < vz - 1; ++stack)
			for (row = stack; row < vy; ++row)
			for (col = row; col < vx; ++col) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col + halfcols, row + halfrows, factor * stack + 1 + halfstacks, state);
						canvas.setCell(row + halfcols, stack + halfrows, factor * col + 1 + halfstacks, state);
						canvas.setCell(stack + halfcols, col + halfrows, factor * row + 1 + halfstacks, state);
						canvas.setCell(-col + halfcols - 1, row + halfrows, factor * stack + 1 + halfstacks, state);
						canvas.setCell(-row + halfcols - 1, stack + halfrows, factor * col + 1 + halfstacks, state);
						canvas.setCell(-stack + halfcols - 1, col + halfrows, factor * row + 1 + halfstacks, state);
					}
			}
			}
			for (stack = -vz; stack < vz; ++stack)
			for (row = stack; row < vy; ++row)
			for (col = row; col < vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col + halfcols, row + halfrows, factor * stack + halfstacks - zDEven)
					&& generator.nextInt(100) < percent) {
				canvas.setCell(col + halfcols, row + halfrows, factor * stack + halfstacks - zDEven, state);
				canvas.setCell(row + halfcols, stack + halfrows, factor * col + halfstacks - zDEven, state);
				canvas.setCell(stack + halfcols, col + halfrows, factor * row + halfstacks - zDEven, state);
				canvas.setCell(-col + halfcols, row + halfrows, factor * stack + halfstacks - zDEven, state);
				canvas.setCell(-row + halfcols, stack + halfrows, factor * col + halfstacks - zDEven, state);
				canvas.setCell(-stack + halfcols, col + halfrows, factor * row + halfstacks - zDEven, state);
			}
			break;
		case 15: /* Full Antisym Cube Random */
			vx = vy = vz = (vx + vy + vz) / 3;
			if (polyhedron == 14) {
				factor = 2;
			for (stack = -vz; stack < -1; ++stack)
			for (row = stack; row < vy; ++row)
			for (col = row; col < vx; ++col) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col + halfcols, row + halfrows, factor * stack + 1 + halfstacks, state);
						canvas.setCell(row + halfcols, stack + halfrows, factor * col + 1 + halfstacks, state);
						canvas.setCell(stack + halfcols, col + halfrows, factor * row + 1 + halfstacks, state);
						canvas.setCell(-col + halfcols - 1, -row + halfrows - 1, -factor * stack - 1 + halfstacks, state);
						canvas.setCell(-row + halfcols - 1, -stack + halfrows - 1, -factor * col - 1 + halfstacks, state);
						canvas.setCell(-stack + halfcols - 1, -col + halfrows - 1, -factor * row - 1 + halfstacks, state);
					}
			}
			}
			/*int tri = generator.nextInt(2);
			tri = 1;
			if (tri == 1 && polyhedron == 12) {
			for (stack = -vz; stack < 0; ++stack)
			for (row = stack; row < vy; ++row)
			for (col = row; col < vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col + halfcols + even + tri, row + halfrows + even + tri, factor * stack + halfstacks + even, state);
						canvas.setCell(row + halfcols + even + tri, stack + halfrows + even, factor * col + halfstacks + even + tri, state);
						canvas.setCell(stack + halfcols + even, col + halfrows + even + tri, factor * row + halfstacks + even + tri, state);
						canvas.setCell(-col + halfcols + tri, -row + halfrows + tri, -factor * stack + halfstacks, state);
						canvas.setCell(-row + halfcols + tri, -stack + halfrows, -factor * col + halfstacks + tri, state);
						canvas.setCell(-stack + halfcols, -col + halfrows + tri, -factor * row + halfstacks + tri, state);
					}
			}
			} else { */
			for (stack = -vz; stack < 0; ++stack)
			for (row = stack; row < vy; ++row)
			for (col = row; col < vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even - zDEven)
					&& generator.nextInt(100) < percent) {
				canvas.setCell(col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even - zDEven, state);
				canvas.setCell(row + halfcols + even, stack + halfrows + even, factor * col + halfstacks + even - zDEven, state);
				canvas.setCell(stack + halfcols + even, col + halfrows + even, factor * row + halfstacks + even - zDEven, state);
				canvas.setCell(-col + halfcols, -row + halfrows, -factor * stack + halfstacks - zDEven, state);
				canvas.setCell(-row + halfcols, -stack + halfrows, -factor * col + halfstacks - zDEven, state);
				canvas.setCell(-stack + halfcols, -col + halfrows, -factor * row + halfstacks - zDEven, state);
			}
			if (Life3DCanvas.cellCheck(polyhedron, halfcols, halfrows, halfstacks - zDEven)
					&& even == 0 && generator.nextInt(100) < percent) {
				canvas.setCell(halfcols, halfrows, halfstacks - zDEven, state);
			}
			break;
		case 16: /* Full Sym Cube Random */
			vz = (vx + vy + vz) / 3;
			if (polyhedron == 14) {
				factor = 2;
			for (stack = -vz; stack < -1; ++stack)
			for (row = stack; row <= 0; ++row)
			for (col = row; col <= 0; ++col) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col + halfcols, row + halfrows, factor * stack + 1 + halfstacks, state);
						canvas.setCell(row + halfcols, stack + halfrows, factor * col + 1 + halfstacks, state);
						canvas.setCell(stack + halfcols, col + halfrows, factor * row + 1 + halfstacks, state);
						canvas.setCell(-col + halfcols - 1, -row + halfrows - 1, -factor * stack - 1 + halfstacks, state);
						canvas.setCell(-row + halfcols - 1, -stack + halfrows - 1, -factor * col - 1 + halfstacks, state);
						canvas.setCell(-stack + halfcols - 1, -col + halfrows - 1, -factor * row - 1 + halfstacks, state);
						canvas.setCell(-col + halfcols - 1, row + halfrows, factor * stack + 1 + halfstacks, state);
						canvas.setCell(-row + halfcols - 1, stack + halfrows, factor * col + 1 + halfstacks, state);
						canvas.setCell(-stack + halfcols - 1, col + halfrows, factor * row + 1 + halfstacks, state);
						canvas.setCell(col + halfcols, -row + halfrows - 1, -factor * stack - 1 + halfstacks, state);
						canvas.setCell(row + halfcols, -stack + halfrows - 1, -factor * col - 1 + halfstacks, state);
						canvas.setCell(stack + halfcols, -col + halfrows - 1, -factor * row - 1 + halfstacks, state);
						canvas.setCell(col + halfcols, -row + halfrows - 1, factor * stack + 1 + halfstacks, state);
						canvas.setCell(row + halfcols, -stack + halfrows - 1, factor * col + 1 + halfstacks, state);
						canvas.setCell(stack + halfcols, -col + halfrows - 1, factor * row + 1 + halfstacks, state);
						canvas.setCell(-col + halfcols - 1, row + halfrows, -factor * stack - 1 + halfstacks, state);
						canvas.setCell(-row + halfcols - 1, stack + halfrows, -factor * col - 1 + halfstacks, state);
						canvas.setCell(-stack + halfcols - 1, col + halfrows, -factor * row - 1 + halfstacks, state);
						canvas.setCell(col + halfcols, row + halfrows, -factor * stack - 1 + halfstacks, state);
						canvas.setCell(row + halfcols, stack + halfrows, -factor * col - 1 + halfstacks, state);
						canvas.setCell(stack + halfcols, col + halfrows, -factor * row - 1 + halfstacks, state);
						canvas.setCell(-col + halfcols - 1, -row + halfrows - 1, factor * stack + 1 + halfstacks, state);
						canvas.setCell(-row + halfcols - 1, -stack + halfrows - 1, factor * col + 1 + halfstacks, state);
						canvas.setCell(-stack + halfcols - 1, -col + halfrows - 1, factor * row + 1 + halfstacks, state);
					}
			}
			}
			/*if (polyhedron == 12) {
				yEven = generator.nextInt(2);
				zEven = generator.nextInt(2);
			} else {
				yEven = 0;
				zEven = 1;
			}*/
			for (stack = -vz; stack < 0; ++stack)
			for (row = stack; row <= 0; ++row)
			for (col = row; col <= 0; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even - zDEven)
					&& generator.nextInt(100) < percent) {
				canvas.setCell(col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even - zDEven, state);
				canvas.setCell(row + halfcols + even, stack + halfrows + even, factor * col + halfstacks + even - zDEven, state);
				canvas.setCell(stack + halfcols + even, col + halfrows + even, factor * row + halfstacks + even - zDEven, state);
				canvas.setCell(col + halfcols + even, -row + halfrows, -factor * stack + halfstacks - zDEven, state);
				canvas.setCell(row + halfcols + even, -stack + halfrows, -factor * col + halfstacks - zDEven, state);
				canvas.setCell(stack + halfcols + even, -col + halfrows, -factor * row + halfstacks - zDEven, state);
				canvas.setCell(-col + halfcols, row + halfrows + even, -factor * stack + halfstacks - zDEven, state);
				canvas.setCell(-row + halfcols, stack + halfrows + even, -factor * col + halfstacks - zDEven, state);
				canvas.setCell(-stack + halfcols, col + halfrows + even, -factor * row + halfstacks - zDEven, state);
				canvas.setCell(-col + halfcols, -row + halfrows, factor * stack + halfstacks + even - zDEven, state);
				canvas.setCell(-row + halfcols, -stack + halfrows, factor * col + halfstacks + even - zDEven, state);
				canvas.setCell(-stack + halfcols, -col + halfrows, factor * row + halfstacks + even - zDEven, state);
				/*if (yEven == 1) {*/
				canvas.setCell(row + halfcols + even, col + halfrows + even, factor * stack + halfstacks + even - zDEven, state);
				canvas.setCell(col + halfcols + even, stack + halfrows + even, factor * row + halfstacks + even - zDEven, state);
				canvas.setCell(stack + halfcols + even, row + halfrows + even, factor * col + halfstacks + even - zDEven, state);
				canvas.setCell(row + halfcols + even, -col + halfrows, -factor * stack + halfstacks - zDEven, state);
				canvas.setCell(col + halfcols + even, -stack + halfrows, -factor * row + halfstacks - zDEven, state);
				canvas.setCell(stack + halfcols + even, -row + halfrows, -factor * col + halfstacks - zDEven, state);
				canvas.setCell(-row + halfcols, col + halfrows + even, -factor * stack + halfstacks - zDEven, state);
				canvas.setCell(-col + halfcols, stack + halfrows + even, -factor * row + halfstacks - zDEven, state);
				canvas.setCell(-stack + halfcols, row + halfrows + even, -factor * col + halfstacks - zDEven, state);
				canvas.setCell(-row + halfcols, -col + halfrows, factor * stack + halfstacks + even - zDEven, state);
				canvas.setCell(-col + halfcols, -stack + halfrows, factor * row + halfstacks + even - zDEven, state);
				canvas.setCell(-stack + halfcols, -row + halfrows, factor * col + halfstacks + even - zDEven, state);
				/*}*/
				/*if (zEven == 1) {*/
				canvas.setCell(-col + halfcols, -row + halfrows, -factor * stack + halfstacks - zDEven, state);
				canvas.setCell(-row + halfcols, -stack + halfrows, -factor * col + halfstacks - zDEven, state);
				canvas.setCell(-stack + halfcols, -col + halfrows, -factor * row + halfstacks - zDEven, state);
				canvas.setCell(-col + halfcols, row + halfrows + even, factor * stack + halfstacks + even - zDEven, state);
				canvas.setCell(-row + halfcols, stack + halfrows + even, factor * col + halfstacks + even - zDEven, state);
				canvas.setCell(-stack + halfcols, col + halfrows + even, factor * row + halfstacks + even - zDEven, state);
				canvas.setCell(col + halfcols + even, -row + halfrows, factor * stack + halfstacks + even - zDEven, state);
				canvas.setCell(row + halfcols + even, -stack + halfrows, factor * col + halfstacks + even - zDEven, state);
				canvas.setCell(stack + halfcols + even, -col + halfrows, factor * row + halfstacks + even - zDEven, state);
				canvas.setCell(col + halfcols + even, row + halfrows + even, -factor * stack + halfstacks - zDEven, state);
				canvas.setCell(row + halfcols + even, stack + halfrows + even, -factor * col + halfstacks - zDEven, state);
				canvas.setCell(stack + halfcols + even, col + halfrows + even, -factor * row + halfstacks - zDEven, state);
				/*if (yEven == 1) {*/
				canvas.setCell(-row + halfcols, -col + halfrows, -factor * stack + halfstacks - zDEven, state);
				canvas.setCell(-col + halfcols, -stack + halfrows, -factor * row + halfstacks - zDEven, state);
				canvas.setCell(-stack + halfcols, -row + halfrows, -factor * col + halfstacks - zDEven, state);
				canvas.setCell(-row + halfcols, col + halfrows + even, factor * stack + halfstacks + even - zDEven, state);
				canvas.setCell(-col + halfcols, stack + halfrows + even, factor * row + halfstacks + even - zDEven, state);
				canvas.setCell(-stack + halfcols, row + halfrows + even, factor * col + halfstacks + even - zDEven, state);
				canvas.setCell(row + halfcols + even, -col + halfrows, factor * stack + halfstacks + even - zDEven, state);
				canvas.setCell(col + halfcols + even, -stack + halfrows, factor * row + halfstacks + even - zDEven, state);
				canvas.setCell(stack + halfcols + even, -row + halfrows, factor * col + halfstacks + even - zDEven, state);
				canvas.setCell(row + halfcols + even, col + halfrows + even, -factor * stack + halfstacks - zDEven, state);
				canvas.setCell(col + halfcols + even, stack + halfrows + even, -factor * row + halfstacks - zDEven, state);
				canvas.setCell(stack + halfcols + even, row + halfrows + even, -factor * col + halfstacks - zDEven, state);
				/*}*/
				/*}*/
			}
			if (Life3DCanvas.cellCheck(polyhedron, halfcols, halfrows, halfstacks - zDEven)
					&& even == 0 && generator.nextInt(100) < percent) {
				canvas.setCell(halfcols, halfrows, halfstacks - zDEven, state);
			}
			break;
		case 17: /* Full Tetra Random */
			vz = (vx + vy + vz) / 3;
			if (polyhedron == 14) {
				factor = 2;
			for (stack = -vz; stack < -1; ++stack)
			for (row = stack; row <= 0; ++row)
			for (col = row; col <= 0; ++col) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col + halfcols, row + halfrows, factor * stack + 1 + halfstacks, state);
						canvas.setCell(row + halfcols, stack + halfrows, factor * col + 1 + halfstacks, state);
						canvas.setCell(stack + halfcols, col + halfrows, factor * row + 1 + halfstacks, state);
						canvas.setCell(col + halfcols, -row + halfrows - 1, -factor * stack - 1 + halfstacks, state);
						canvas.setCell(row + halfcols, -stack + halfrows - 1, -factor * col - 1 + halfstacks, state);
						canvas.setCell(stack + halfcols, -col + halfrows - 1, -factor * row - 1 + halfstacks, state);
						canvas.setCell(-col + halfcols - 1, row + halfrows, -factor * stack - 1 + halfstacks, state);
						canvas.setCell(-row + halfcols - 1, stack + halfrows, -factor * col - 1 + halfstacks, state);
						canvas.setCell(-stack + halfcols - 1, col + halfrows, -factor * row - 1 + halfstacks, state);
						canvas.setCell(-col + halfcols - 1, -row + halfrows - 1, factor * stack + 1 + halfstacks, state);
						canvas.setCell(-row + halfcols - 1, -stack + halfrows - 1, factor * col + 1 + halfstacks, state);
						canvas.setCell(-stack + halfcols - 1, -col + halfrows - 1, factor * row + 1 + halfstacks, state);
					}
			}
			}
			for (stack = -vz; stack < 0; ++stack)
			for (row = stack; row <= 0; ++row)
			for (col = row; col <= 0; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even - zDEven)
					&& generator.nextInt(100) < percent) {
				canvas.setCell(col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even - zDEven, state);
				canvas.setCell(row + halfcols + even, stack + halfrows + even, factor * col + halfstacks + even - zDEven, state);
				canvas.setCell(stack + halfcols + even, col + halfrows + even, factor * row + halfstacks + even - zDEven, state);
				canvas.setCell(col + halfcols + even, -row + halfrows, -factor * stack + halfstacks - zDEven, state);
				canvas.setCell(row + halfcols + even, -stack + halfrows, -factor * col + halfstacks - zDEven, state);
				canvas.setCell(stack + halfcols + even, -col + halfrows, -factor * row + halfstacks - zDEven, state);
				canvas.setCell(-col + halfcols, row + halfrows + even, -factor * stack + halfstacks - zDEven, state);
				canvas.setCell(-row + halfcols, stack + halfrows + even, -factor * col + halfstacks - zDEven, state);
				canvas.setCell(-stack + halfcols, col + halfrows + even, -factor * row + halfstacks - zDEven, state);
				canvas.setCell(-col + halfcols, -row + halfrows, factor * stack + halfstacks + even - zDEven, state);
				canvas.setCell(-row + halfcols, -stack + halfrows, factor * col + halfstacks + even - zDEven, state);
				canvas.setCell(-stack + halfcols, -col + halfrows, factor * row + halfstacks + even - zDEven, state);
			}
			if (Life3DCanvas.cellCheck(polyhedron, halfcols, halfrows, halfstacks - zDEven)
					&& even == 0 && generator.nextInt(100) < percent) {
				canvas.setCell(halfcols, halfrows, halfstacks - zDEven, state);
			}
			break;
		case 18: /* Alt Full Antisym Cube Random */
			vz = (vx + vy + vz) / 3;
			if (polyhedron == 14) {
				factor = 2;
			for (stack = -vz; stack < -1; ++stack)
			for (row = stack; row < vy; ++row)
			for (col = row; col < vx; ++col) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col + halfcols, row + halfrows, factor * stack + 1 + halfstacks, state);
						canvas.setCell(row + halfcols, stack + halfrows, factor * col + 1 + halfstacks, state);
						canvas.setCell(stack + halfcols, col + halfrows, factor * row + 1 + halfstacks, state);
						canvas.setCell(-col + halfcols - 1, -row + halfrows - 1, -factor * stack - 1 + halfstacks, state);
						canvas.setCell(-row + halfcols - 1, -stack + halfrows - 1, -factor * col - 1 + halfstacks, state);
						canvas.setCell(-stack + halfcols - 1, -col + halfrows - 1, -factor * row - 1 + halfstacks, state);
					}
			}
			}
			/*int tri = generator.nextInt(2);
			tri = 1;
			if (tri == 1 && polyhedron == 12) {
			for (stack = -vz; stack < 0; ++stack)
			for (row = stack; row < vy; ++row)
			for (col = row; col < vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col + halfcols + even + tri, row + halfrows + even + tri, factor * stack + halfstacks + even, state);
						canvas.setCell(row + halfcols + even + tri, stack + halfrows + even, factor * col + halfstacks + even + tri, state);
						canvas.setCell(stack + halfcols + even, col + halfrows + even + tri, factor * row + halfstacks + even + tri, state);
						canvas.setCell(-col + halfcols + tri, -row + halfrows + tri, -factor * stack + halfstacks, state);
						canvas.setCell(-row + halfcols + tri, -stack + halfrows, -factor * col + halfstacks + tri, state);
						canvas.setCell(-stack + halfcols, -col + halfrows + tri, -factor * row + halfstacks + tri, state);
					}
			}
			} else { */
			for (stack = -vz; stack < 0; ++stack)
			for (row = stack; row < vy; ++row)
			for (col = row; col < vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even - zDEven)
					&& generator.nextInt(100) < percent) {
				canvas.setCell(col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even - zDEven, state);
				canvas.setCell(row + halfcols + even, stack + halfrows + even, factor * col + halfstacks + even - zDEven, state);
				canvas.setCell(stack + halfcols + even, col + halfrows + even, factor * row + halfstacks + even - zDEven, state);
				canvas.setCell(-col + halfcols, -stack + halfrows, -factor * row + halfstacks - zDEven, state);
				canvas.setCell(-row + halfcols, -col + halfrows, -factor * stack + halfstacks - zDEven, state);
				canvas.setCell(-stack + halfcols, -row + halfrows, -factor * col + halfstacks - zDEven, state);
			}
			if (Life3DCanvas.cellCheck(polyhedron, halfcols, halfrows, halfstacks - zDEven)
					&& even == 0 && generator.nextInt(100) < percent) {
				canvas.setCell(halfcols, halfrows, halfstacks - zDEven, state);
			}
			break;
		case 19: /* Propeller Random */
			for (stack = halfstacks - vz; stack < halfstacks + vz; ++stack)
			for (row = halfrows - vy; row < halfrows + vy; ++row)
			for (col = halfcols; col < halfcols + vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)) {
					if (polyhedron == 14) {
						zEven = yEven = even = (stack & 1);
					}
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col, row, stack, state);
						canvas.setCell(2 * halfcols - col - even, 2 * halfrows - row - yEven,
							2 * halfstacks - stack - zEven, state);
					}
			}
			break;
		case 20: /* New One Random */
			/*for (stack = halfstacks - vz; stack < halfstacks + vz; ++stack)
			for (row = halfrows; row < halfrows + vy; ++row)
			for (col = halfcols - vx; col < halfcols + vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col, row, stack, state);
						canvas.setCell(col, 2 * halfrows - row - yEven, stack, state);
					}
			}
			break;*/
			// Diagonal
			/*vy = vz = (vz + vy) / 2;*/
			/*for (stack = -vz; stack < vz; ++stack)
			for (row = stack; row < vy; ++row)
			for (col = halfcols; col < halfcols + vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col, row + halfrows, stack + halfstacks, state);
						canvas.setCell(2 * halfcols - col - even, halfrows - row - yEven, stack + halfstacks, state);
						canvas.setCell(col, stack + halfrows, row + halfstacks, state);
						canvas.setCell(2 * halfcols - col - even, halfrows - stack - yEven, row + halfstacks, state);
					}
			}*/
			// Diagonal Antisym/sym
			/*vx = vy = (vx + vy) / 2;
			for (stack = halfstacks - vz; stack < halfstacks; ++stack)
			for (row = -vy; row < vy; ++row)
			for (col = row; col < vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)) {
					if (generator.nextInt(100) < percent) {
						if (even == 1) {
							canvas.setCell(col + halfcols, row + halfrows, stack, state);
							canvas.setCell(row + halfcols, col + halfrows, stack, state);
							canvas.setCell(col + halfcols, row + halfrows, 2 * halfstacks - stack, state);
							canvas.setCell(row + halfcols, col + halfrows, 2 * halfstacks - stack, state);
						} else {
							canvas.setCell(-col + halfcols, row + halfrows, stack, state);
							canvas.setCell(-row + halfcols, col + halfrows, stack, state);
							canvas.setCell(-col + halfcols, row + halfrows, 2 * halfstacks - stack, state);
							canvas.setCell(-row + halfcols, col + halfrows, 2 * halfstacks - stack, state);
						}
					}
			}*/
			/*for (stack = -vz; stack < 0; ++stack)
			for (row = stack; row <= 0; ++row)
			for (col = row; col <= 0; ++col) {
			if (Life3DCanvas.cellCheck(polyhedron, col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even)) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even, state);
						canvas.setCell(row + halfcols + even, stack + halfrows + even, factor * col + halfstacks + even, state);
						//canvas.setCell(stack + halfcols + even, col + halfrows + even, factor * row + halfstacks + even, state);
						canvas.setCell(col + halfcols + even, -row + halfrows, -factor * stack + halfstacks, state);
						canvas.setCell(row + halfcols + even, -stack + halfrows, -factor * col + halfstacks, state);
						//canvas.setCell(stack + halfcols + even, -col + halfrows, -factor * row + halfstacks, state);
						canvas.setCell(-col + halfcols, row + halfrows + even, -factor * stack + halfstacks, state);
						canvas.setCell(-row + halfcols, stack + halfrows + even, -factor * col + halfstacks, state);
						//canvas.setCell(-stack + halfcols, col + halfrows + even, -factor * row + halfstacks, state);
						canvas.setCell(-col + halfcols, -row + halfrows, factor * stack + halfstacks + even, state);
						canvas.setCell(-row + halfcols, -stack + halfrows, factor * col + halfstacks + even, state);
						//canvas.setCell(-stack + halfcols, -col + halfrows, factor * row + halfstacks + even, state);
					}
			}
			}*/
			// Trying for non tetrahedral trisymmetry from #16
			/*vx = vy = vz = (vx + vy + vz) / 2;
			for (stack = -vz; stack < vz; ++stack)
			for (row = stack; row <= 0; ++row)
			for (col = row; col <= 0; ++col) {
			if (Life3DCanvas.cellCheck(polyhedron, col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even)) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even, state);
						canvas.setCell(row + halfcols + even, stack + halfrows + even, factor * col + halfstacks + even, state);
						canvas.setCell(stack + halfcols + even, col + halfrows + even, factor * row + halfstacks + even, state);
						canvas.setCell(row + halfcols + even, col + halfrows + even, factor * stack + halfstacks + even, state);
						canvas.setCell(col + halfcols + even, stack + halfrows + even, factor * row + halfstacks + even, state);
						canvas.setCell(stack + halfcols + even, row + halfrows + even, factor * col + halfstacks + even, state);
						canvas.setCell(-col + 2 * halfcols + even, -row + 2 * halfrows + even, factor * stack + halfstacks + even, state);
						canvas.setCell(-row + 2 * halfcols + even, -stack + 2 * halfrows + even, factor * col + halfstacks + even, state);
						canvas.setCell(-stack + 2 * halfcols + even, -col + 2 * halfrows + even, factor * row + halfstacks + even, state);
						canvas.setCell(-row + 2 * halfcols + even, -col + 2 * halfrows + even, factor * stack + halfstacks + even, state);
						canvas.setCell(-col + 2 * halfcols + even, -stack + 2 * halfrows + even, factor * row + halfstacks + even, state);
						canvas.setCell(-stack + 2 * halfcols + even, -row + 2 * halfrows + even, factor * col + halfstacks + even, state);
					}
			}
			}*/
			/* This was an attempt at 6H2, not sure
			vx = vy = (vx + vy) / 2;
			for (stack = halfstacks - vz; stack < halfstacks; ++stack)
			for (row = halfrows - vy; row < halfrows + vy; ++row)
			for (col = halfcols; col < halfcols + vx; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col, row, stack)) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col, row, stack, state);
						canvas.setCell(row, 2 * halfrows - col, 2 * halfstacks - stack, state);
						//canvas.setCell(2 * halfcols - row - even, col, 2 * halfstacks - stack, state);
					}
			}*/
			/*for (stack = -vz; stack < 0; ++stack)
			for (row = stack; row <= 0; ++row)
			for (col = row; col <= 0; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even)) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even, state);
						canvas.setCell(row + halfcols + even, stack + halfrows + even, factor * col + halfstacks + even, state);
						canvas.setCell(stack + halfcols + even, col + halfrows + even, factor * row + halfstacks + even, state);
						canvas.setCell(row + halfcols + even, col + halfrows + even, factor * stack + halfstacks + even, state);
						canvas.setCell(col + halfcols + even, stack + halfrows + even, factor * row + halfstacks + even, state);
						canvas.setCell(stack + halfcols + even, row + halfrows + even, factor * col + halfstacks + even, state);

						canvas.setCell(col + halfcols + even, -row + halfrows, -factor * stack + halfstacks, state);
						canvas.setCell(row + halfcols + even, -stack + halfrows, -factor * col + halfstacks, state);
						canvas.setCell(stack + halfcols + even, -col + halfrows, -factor * row + halfstacks, state);
						canvas.setCell(row + halfcols + even, -col + halfrows, -factor * stack + halfstacks, state);
						canvas.setCell(col + halfcols + even, -stack + halfrows, -factor * row + halfstacks, state);
						canvas.setCell(stack + halfcols + even, -row + halfrows, -factor * col + halfstacks, state);

						canvas.setCell(-col + halfcols, row + halfrows + even, -factor * stack + halfstacks, state);
						canvas.setCell(-row + halfcols, stack + halfrows + even, -factor * col + halfstacks, state);
						canvas.setCell(-stack + halfcols, col + halfrows + even, -factor * row + halfstacks, state);
						canvas.setCell(-row + halfcols, col + halfrows + even, -factor * stack + halfstacks, state);
						canvas.setCell(-col + halfcols, stack + halfrows + even, -factor * row + halfstacks, state);
						canvas.setCell(-stack + halfcols, row + halfrows + even, -factor * col + halfstacks, state);

						canvas.setCell(-col + halfcols, -row + halfrows, factor * stack + halfstacks + even, state);
						canvas.setCell(-row + halfcols, -stack + halfrows, factor * col + halfstacks + even, state);
						canvas.setCell(-stack + halfcols, -col + halfrows, factor * row + halfstacks + even, state);
						canvas.setCell(-row + halfcols, -col + halfrows, factor * stack + halfstacks + even, state);
						canvas.setCell(-col + halfcols, -stack + halfrows, factor * row + halfstacks + even, state);
						canvas.setCell(-stack + halfcols, -row + halfrows, factor * col + halfstacks + even, state);
					}
			}
			for (stack = -vz; stack < 0; ++stack)
			for (row = stack; row <= 0; ++row)
			for (col = row; col <= 0; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even)) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(-col + halfcols, -row + halfrows, -factor * stack + halfstacks, state);
						canvas.setCell(-row + halfcols, -stack + halfrows, -factor * col + halfstacks, state);
						canvas.setCell(-stack + halfcols, -col + halfrows, -factor * row + halfstacks, state);
						canvas.setCell(-row + halfcols, -col + halfrows, -factor * stack + halfstacks, state);
						canvas.setCell(-col + halfcols, -stack + halfrows, -factor * row + halfstacks, state);
						canvas.setCell(-stack + halfcols, -row + halfrows, -factor * col + halfstacks, state);

						canvas.setCell(-col + halfcols, row + halfrows + even, factor * stack + halfstacks + even, state);
						canvas.setCell(-row + halfcols, stack + halfrows + even, factor * col + halfstacks + even, state);
						canvas.setCell(-stack + halfcols, col + halfrows + even, factor * row + halfstacks + even, state);
						canvas.setCell(-row + halfcols, col + halfrows + even, factor * stack + halfstacks + even, state);
						canvas.setCell(-col + halfcols, stack + halfrows + even, factor * row + halfstacks + even, state);
						canvas.setCell(-stack + halfcols, row + halfrows + even, factor * col + halfstacks + even, state);

						canvas.setCell(col + halfcols + even, -row + halfrows, factor * stack + halfstacks + even, state);
						canvas.setCell(row + halfcols + even, -stack + halfrows, factor * col + halfstacks + even, state);
						canvas.setCell(stack + halfcols + even, -col + halfrows, factor * row + halfstacks + even, state);
						canvas.setCell(row + halfcols + even, -col + halfrows, factor * stack + halfstacks + even, state);
						canvas.setCell(col + halfcols + even, -stack + halfrows, factor * row + halfstacks + even, state);
						canvas.setCell(stack + halfcols + even, -row + halfrows, factor * col + halfstacks + even, state);

						canvas.setCell(col + halfcols + even, row + halfrows + even, -factor * stack + halfstacks, state);
						canvas.setCell(row + halfcols + even, stack + halfrows + even, -factor * col + halfstacks, state);
						canvas.setCell(stack + halfcols + even, col + halfrows + even, -factor * row + halfstacks, state);
						canvas.setCell(row + halfcols + even, col + halfrows + even, -factor * stack + halfstacks, state);
						canvas.setCell(col + halfcols + even, stack + halfrows + even, -factor * row + halfstacks, state);
						canvas.setCell(stack + halfcols + even, row + halfrows + even, -factor * col + halfstacks, state);
					}
			}
			if (Life3DCanvas.cellCheck(polyhedron, halfcols, halfrows, halfstacks)) {
				if (even == 0 && generator.nextInt(100) < percent) {
					canvas.setCell(halfcols, halfrows, halfstacks, state);
				}
			}*/
			/*for (stack = -vz; stack < 0; ++stack)
			for (row = stack; row <= 0; ++row)
			for (col = row; col <= 0; ++col) {
			if (Life3DCanvas.cellCheck(polyhedron, col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even)) {
					if (generator.nextInt(100) < percent) {
						canvas.setCell(col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even, state);
						canvas.setCell(row + halfcols + even, stack + halfrows + even, factor * col + halfstacks + even, state);
						canvas.setCell(stack + halfcols + even, col + halfrows + even, factor * row + halfstacks + even, state);

						canvas.setCell(row + halfcols + even, col + halfrows + even, factor * stack + halfstacks + even, state);
						canvas.setCell(col + halfcols + even, stack + halfrows + even, factor * row + halfstacks + even, state);
						canvas.setCell(stack + halfcols + even, row + halfrows + even, factor * col + halfstacks + even, state);
					}
			}
			}*/
			offset = generator.nextInt(2);
			if (polyhedron == 4 || polyhedron == 8 || polyhedron == 16)
				offset *= 2;
			for (stack = -vz; stack < 0; ++stack)
			for (row = stack; row <= 0; ++row)
			for (col = row; col <= 0; ++col)
			if (Life3DCanvas.cellCheck(polyhedron, col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even)
					&& generator.nextInt(100) < percent) {
				canvas.setCell(col + halfcols + even, row + halfrows + even, factor * stack + halfstacks + even, state);
				canvas.setCell(row + halfcols + even, stack + halfrows + even, factor * col + halfstacks + even, state);
				canvas.setCell(stack + halfcols + even, col + halfrows + even, factor * row + halfstacks + even, state);
				canvas.setCell(row + halfcols + even, col + halfrows + even, factor * stack + halfstacks + even, state);
				canvas.setCell(col + halfcols + even, stack + halfrows + even, factor * row + halfstacks + even, state);
				canvas.setCell(stack + halfcols + even, row + halfrows + even, factor * col + halfstacks + even, state);

				canvas.setCell(col + halfcols + even, -row + halfrows - offset, -factor * stack + halfstacks - offset, state);
				canvas.setCell(row + halfcols + even, -stack + halfrows - offset, -factor * col + halfstacks - offset, state);
				canvas.setCell(stack + halfcols + even, -col + halfrows - offset, -factor * row + halfstacks - offset, state);
				canvas.setCell(row + halfcols + even, -col + halfrows - offset, -factor * stack + halfstacks - offset, state);
				canvas.setCell(col + halfcols + even, -stack + halfrows - offset, -factor * row + halfstacks - offset, state);
				canvas.setCell(stack + halfcols + even, -row + halfrows - offset, -factor * col + halfstacks - offset, state);

				canvas.setCell(-col + halfcols - offset, row + halfrows + even, -factor * stack + halfstacks - offset, state);
				canvas.setCell(-row + halfcols - offset, stack + halfrows + even, -factor * col + halfstacks - offset, state);
				canvas.setCell(-stack + halfcols - offset, col + halfrows + even, -factor * row + halfstacks - offset, state);
				canvas.setCell(-row + halfcols - offset, col + halfrows + even, -factor * stack + halfstacks - offset, state);
				canvas.setCell(-col + halfcols - offset, stack + halfrows + even, -factor * row + halfstacks - offset, state);
				canvas.setCell(-stack + halfcols - offset, row + halfrows + even, -factor * col + halfstacks - offset, state);

				canvas.setCell(-col + halfcols - offset, -row + halfrows - offset, factor * stack + halfstacks + even, state);
				canvas.setCell(-row + halfcols - offset, -stack + halfrows - offset, factor * col + halfstacks + even, state);
				canvas.setCell(-stack + halfcols - offset, -col + halfrows - offset, factor * row + halfstacks + even, state);
				canvas.setCell(-row + halfcols - offset, -col + halfrows - offset, factor * stack + halfstacks + even, state);
				canvas.setCell(-col + halfcols - offset, -stack + halfrows - offset, factor * row + halfstacks + even, state);
				canvas.setCell(-stack + halfcols - offset, -row + halfrows - offset, factor * col + halfstacks + even, state);
			}
			break;
		default:
			System.out.println("wrong symmetry index " +
				symmetryIndex);
		}
	}
}
