package com.sillycycle.bagleyd.util;

/*
 * @(#)Cell.java
 *
 * Copyright 2009 - 2014  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.Color;
import java.awt.Component;
import java.awt.Point;

/**
 * The <code>Cell</code> class handles the construction of the cells.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/life.html
 */

public class Cell extends Component {
	private static final long serialVersionUID = 42L;

	CellCb cellCb = null;
	CellDd cellDd = null;
	CellTd cellTd = null;
	CellAc cellAc = null;

	int polyhedron; /* faces of polyhedron = 6, 12, 14, 16, 22 */

	public Cell(int polyhedron, int nx, int ny/*, int nz*/,
			int screenWidth, int screenHeight) {
		cellCb = null;
		cellDd = null;
		cellTd = null;
		cellAc = null;
		this.polyhedron = polyhedron;
		if (polyhedron == 6) {
			cellCb = new CellCb(nx, ny/*, nz*/,
				screenWidth, screenHeight);
		} else if (polyhedron == 12) {
			cellDd = new CellDd(nx, ny/*, nz*/,
				screenWidth, screenHeight);
		} else if (polyhedron == 14) {
			cellTd = new CellTd(nx, ny/*, nz*/,
				screenWidth, screenHeight);
		} else if (polyhedron == 16 || polyhedron == 22) {
			cellAc = new CellAc(nx, ny/*, nz*/,
				screenWidth, screenHeight);
		}
	}

	/*Box boundBox(int nx, int ny, int nz) {
		if (polyhedron == 6) {
			return cellCb.boundBox(nx, ny, nz);
		} else if (polyhedron == 12) {
			return cellDd.boundBox(nx, ny, nz);
		} else if (polyhedron == 14) {
			return cellTd.boundBox(nx, ny, nz);
		} else if (polyhedron == 16 || polyhedron == 22) {
			return cellAc.boundBox(nx, ny, nz);
		}
		return null;
	}*/

	public Point3D screenToCell(int x, int y, int z) {
		if (polyhedron == 6) {
			return cellCb.screenToCell(x, y, z);
		} else if (polyhedron == 12) {
			return cellDd.screenToCell(x, y, z);
		} else if (polyhedron == 14) {
			return cellTd.screenToCell(x, y, z);
		} else if (polyhedron == 16 || polyhedron == 22) {
			return cellAc.screenToCell(x, y, z);
		}
		return null;
	}

	public boolean inCell(Graphics3D g, Point pt, int i, int j, int k) {
		if (g == null)
			return false;
		if (polyhedron == 6) {
			return CellCb.inCell(g, pt, i, j, k);
		} else if (polyhedron == 12) {
			return CellDd.inCell(g, pt, i, j, k);
		} else if (polyhedron == 14) {
			return CellTd.inCell(g, pt, i, j, k);
		} else if (polyhedron == 16 || polyhedron == 22) {
			return CellAc.inCell(g, pt, i, j, k);
		}
		return false;
	}

	public void drawCell(Graphics3D g, boolean round, boolean wireframe,
			Color cid, Color bid,
			int i, int j, int k) {
		if (g == null)
			return;
		if (polyhedron == 6) {
			CellCb.drawCell(g, round, wireframe, cid, bid, i, j, k);
		} else if (polyhedron == 12) {
			CellDd.drawCell(g, round, wireframe, cid, bid, i, j, k);
		} else if (polyhedron == 14) {
			CellTd.drawCell(g, round, wireframe, cid, bid, i, j, k);
		} else if (polyhedron == 16 || polyhedron == 22) {
			CellAc.drawCell(g, round, wireframe, cid, bid, i, j, k);
		}
	}

	public void drawCaptureBox(Graphics3D g, Color color,
			int i, int j, int k, int nx, int ny, int nz) {
		if (g == null)
			return;
		if (polyhedron == 6) {
			CellCb.drawCaptureBox(g, color, i, j, k, nx, ny, nz);
		} else if (polyhedron == 12) {
			CellDd.drawCaptureBox(g, color, i, j, k, nx, ny, nz);
		} else if (polyhedron == 14) {
			CellTd.drawCaptureBox(g, color, i, j, k, nx, ny, nz);
		} else if (polyhedron == 16 || polyhedron == 22) {
			CellAc.drawCaptureBox(g, color, i, j, k, nx, ny, nz);
		}
	}
}
