package com.sillycycle.bagleyd.util;

/*
 * @(#)CellAc.java
 *
 * Copyright 2012 - 2014  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;

/**
 * The <code>CellAc</code> class handles
 * the construction of the tetrahedron and octahedron cells.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/life.html
 */

public class CellAc extends Component {
	private static final long serialVersionUID = 42L;

	Dimension d;
	Box b;

	CellAc(int nx, int ny/*, int nz*/,
			int screenWidth, int screenHeight) {
		int xs, ys;
		int width = screenWidth - 4;
		int height = screenHeight - 4;

		if (width <= nx)
			width = nx;
		if (height <= ny)
			height = ny;
		xs = width / nx;
		ys = height / ny;
		d = new Dimension(screenWidth, screenHeight);
		b = new Box(2, 2, 0, xs, ys, (xs * ys) / 4);
	}

	/*Box boundBox(int nx, int ny, int nz) {
		return new Box(0, 0, 0,
			d.width, d.height, b.depth);
	}*/

	Point3D screenToCell(int x, int y, int z) {
		return new Point3D(ModArith.div(x - b.x, b.width),
			ModArith.div(y - b.y, b.height), z);
	}

	static boolean inCell(Graphics3D g, Point pt, int i, int j, int k) {
		Point3D p = new Point3D(i, j, k);

		if (g == null || g.g == null)
			return false;
		return g.inOctahedron(pt, p);
	}

	static void drawCell(Graphics3D g, boolean round, boolean wireframe,
			Color cid, Color bid, int i, int j, int k) {
		Color[] staticColor = new Color[4];
		Color background = bid;
		Point3D p = new Point3D(i, j, k);
		if (g == null || g.g == null)
			return;
		if (cid != null) {
			/*staticColor[0] = Color.red;
			staticColor[1] = Color.green;
			staticColor[2] = Color.blue;*/
			//staticColor[0] = g.darker(cid);
			staticColor[0] = Graphics3D.darker(cid);
			staticColor[1] = cid;
			staticColor[2] = Graphics3D.brighter(cid);
			staticColor[3] = Graphics3D.brighter(staticColor[2]);
			int val = i + j + k;
			if (val % 2 == 0) {
				if (round) {
					g.drawSphere(wireframe, staticColor, bid, p);
				} else {
					g.drawOctahedron(wireframe, staticColor, bid, p);
				}
			} else {
				if (round) {
					g.drawSphere(wireframe, staticColor, bid, p);
				} else {
					if (val % 4 == 1) {
						g.drawTetrahedronTlBr(wireframe, staticColor, bid, p);
					} else {
						g.drawTetrahedronTrBl(wireframe, staticColor, bid, p);
					}
				}
			}
			/*g.setColor(cid);
			g.fillRect(b.x + b.width * i + 1,
				b.y + b.height * j + 1,
				b.width - 1, b.height - 1);*/
		}
		if (cid == null) {
			staticColor[0] = null;
			staticColor[1] = null;
			staticColor[2] = null;
			staticColor[3] = null;
			if (background == null)
				background = Color.black;
			g.drawOctahedron(wireframe, staticColor, background, p);
			//g.drawTetrahedronTrBl(wireframe, staticColor, background, p);
			//g.drawTetrahedronTlBr(wireframe, staticColor, background, p);
				/*g.setColor(bid);
			g.drawCube(g, wireframe, null, bid, p);
			g.drawRect(b.x + b.width * i,
				b.y + b.height * j,
				b.width, b.height);*/
		}
		/*System.out.println("i " + i + ", j " + j + ", k " + k);
		System.out.println("  x " + b.x + ", y " + b.y);
		System.out.println("  width " + b.width + ", height " + b.height);*/
	}

	static void drawCaptureBox(Graphics3D g, Color color,
			int i, int j, int k, int nx, int ny, int nz) {
		if (g == null || g.g == null)
			return;
		g.g.setColor(color);
		for (int depth = k; depth < nz + k; depth++)
		for (int vert = j; vert < ny + j; vert++)
		for (int horiz = i; horiz < nx + i; horiz++) {
			drawCell(g, false, true, null, null, horiz, vert, depth);
		}
	}
}
