package com.sillycycle.bagleyd.util;

/*
 * @(#)Signatures.java
 *
 * Copyright 2009 - 2016  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.util.HashMap;

/**
 * The <code>Signatures</code> class handles the signatures of life.
 * The way implemented should be good for 2 or 3 dimensional life.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/life.html
 */

public class Signatures {
	HashMap<String, Integer> hashtable = new HashMap<>();

	public Signatures() {
		init();
	}

	public static int signatureToNumberLiveCells(String string) {
		int i = 0, live = 0;
		final int BASE = 10;
		int current = 0;
		int multiply = -1;
		if (string == null)
			return 0;
		while (i < string.length() && string.charAt(i) != '<') {
			if (string.charAt(i) >= '0' && string.charAt(i) <= '9') {
				current = current * BASE + string.charAt(i) - '0';
			} else if (string.charAt(i) == '*') {
				multiply = current;
				current = 0;
			} else {
				if (multiply < 0) {
					live += current;
				} else {
					live += current * multiply;
					multiply = -1;
				}
				current = 0;
			}
			i++;
		}
		if (multiply < 0) {
			live += current;
		} else {
			live += current * multiply;
		}
		return live;
	}

	private static String divisible(String string, int divisor) {
		StringBuilder sig = new StringBuilder();
		String[] words = string.split(" ");

//	System.out.println(string + " " + divisor);
try {
		for (int i = 0; i < words.length; i++) {
			String word = words[i];
			if ("<>".equals(word)) {
				sig.append(word).append((i < words.length - 1) ? " " : "");
				continue;
			}
			String[] factors = word.split("\\*");
			int j = 0;
			if (factors.length >= 2)
				sig.append(factors[j++]).append("*");
			int value = Integer.parseInt(factors[j]);
//	System.out.println("divisible: " + value + "+" + (value % divisor) + " " + (value / divisor));
			if ((value % divisor) == 0) {
				sig.append(value / divisor).append((i < words.length - 1) ? " " : "");
			} else {
//				System.out.println("before failure " + sig.toString());
				return null;
			}
		}
} catch (NumberFormatException e) {
			System.out.println("corrupt string:" + string);
			e.printStackTrace();
			return null;
}

//	System.out.println("returning " + sig.toString());
		return sig.toString();
	}

	public int containsSignature(String string) {
//	System.out.println(" contains " + string);
		if (hashtable.containsKey(string))
			return 1;
		// think only have to worry about 2, 3, 4, 6, 8, 12, 16
		String temp2, temp3;

		temp2 = divisible(string, 2);
		if (temp2 != null && hashtable.containsKey(temp2))
			return 2;
		temp3 = divisible(string, 3);
		if (temp3 != null && hashtable.containsKey(temp3))
			return 3;
		if (temp2 == null)
			return 0;
		if (temp3 != null) {
			temp3 = divisible(string, 6);
			if (temp3 != null && hashtable.containsKey(temp3))
				return 6;
		}
		temp2 = divisible(string, 4);
		if (temp2 == null) {
			return 0;
		} else if (hashtable.containsKey(temp2)) {
			return 4;
		}
		temp2 = divisible(string, 8);
		if (temp2 == null) {
			return 0;
		} else if (hashtable.containsKey(temp2)) {
			return 8;
		}
		temp2 = divisible(string, 16);
		if (temp2 != null && hashtable.containsKey(temp2))
			return 16;
		if (temp3 != null) {
			temp2 = divisible(string, 12);
			if (temp2 != null && hashtable.containsKey(temp2))
				return 12;
		}
		return 0;
	}

	void init() {
		if (!hashtable.isEmpty())
			hashtable.clear();
	}

	Integer getSignatureIndex(String string) {
		return hashtable.get(string);
	}

	public void putSignature(String string) {
		hashtable.put(string, hashtable.size());
	}

	void clearSignatures() {
		hashtable.clear();
	}

	public int numberSignature() {
		return hashtable.size();
	}
}
